//
//  OPAppStore.h
//  ApplicationFrameworks
//
//  Created by Roustem Karimov on 2014-07-30.
//
//

#import <Foundation/Foundation.h>
#import "OPApplicationCustomer.h"

FOUNDATION_EXPORT NSString *const OPPremiumMobileProductIdentifier;
FOUNDATION_EXPORT NSString *const OPPremiumDesktopProductIdentifier;

FOUNDATION_EXPORT NSString *const OPIndividualMonthlySubscriptionProductIdentifier;
FOUNDATION_EXPORT NSString *const OPFamilyMonthlySubscriptionProductIdentifier;

FOUNDATION_EXPORT NSString *const OPAppStoreCannotMakePaymentsNotification;
FOUNDATION_EXPORT NSString *const OPAppStoreFailedToLoadProductInformationFromAppStoreNotification;
FOUNDATION_EXPORT NSString *const OPAppStoreCompletedPurchaseNotification;
FOUNDATION_EXPORT NSString *const OPAppStoreLoadedProductInformationFromAppStoreNotification;
FOUNDATION_EXPORT NSString *const OPAppStoreReceivedFreshApplicationReceiptNotification;
FOUNDATION_EXPORT NSString *const OPAppStoreNotificationCompletedPurchaseSuccessfullyKey;
FOUNDATION_EXPORT NSString *const OPAppStoreNotificationCompletedPurchaseProductIdentifierKey;
FOUNDATION_EXPORT NSString *const OPAppStoreNotificationErrorKey;

@class OPAppStore;
@class OPAppStoreProduct;

@interface OPAppStore : NSObject

@property (nonatomic, readonly) NSString *appBundleIdentifier;
@property (nonatomic, readonly) NSString *appBundleVersion;
@property (nonatomic, readonly) BOOL useAppleStore;

@property (nonatomic, readonly) BOOL canPurchase;
@property (nonatomic, readonly, getter=isPurchasingOrRestoring) BOOL purchasingOrRestoring;

+ (BOOL)storeKitCanMakePayments;

- (instancetype)initWithAppBundleIdentifier:(NSString *)appBundleIdentifier appBundleVersion:(NSString *)appBundleVersion useAppleStore:(BOOL)useAppleStore;

- (void)refreshProducts;

- (void)refreshApplicationReceipt;
- (void)startRestoringPurchases;
- (void)performPurchaseOfProduct:(OPAppStoreProduct *)product forViewController:(id)viewController;
- (BOOL)isProductPurchased:(NSString *)productIdentifier;

- (NSMutableDictionary *)purchasedProductsDictionary;

- (OPAppStoreProduct *)appStoreProductForIdentifier:(NSString *)productIdentifier;

- (BOOL)doesProcessedTransactionExistForProductForIdentifier:(NSString *)productIdentifier;

- (void)readApplicationReceiptIfNeeded;

- (NSData *)receiptData; // [Roo, 2016-10-14] This method reads the raw receipt from the file system into an NSData object. It is used to upload the receipt to the B5 server for processing.

@end
